package com.code.aaron.speech;

import java.util.ArrayList;

import android.annotation.TargetApi;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** SpeechPlugin */
@TargetApi(16)
public class SpeechPlugin implements EventChannel.StreamHandler {
    private static final String CHANNEL_NAME = "aaron.code.com/speech";

    /** Plugin registration. */
    public static void registerWith(Registrar registrar) {
    final EventChannel channel = new EventChannel(registrar.messenger(), CHANNEL_NAME);
    channel.setStreamHandler(new SpeechPlugin());
    }

    private EventChannel.EventSink eventSink;
    private String api_key;

    // Audio recorder + initial values
    private static volatile AudioRecord recorder;

    private int AUDIO_SOURCE = MediaRecorder.AudioSource.DEFAULT;
    private int SAMPLE_RATE = 16000;
    private int CHANNEL_CONFIG = AudioFormat.CHANNEL_IN_MONO;
    private int AUDIO_FORMAT = AudioFormat.ENCODING_PCM_16BIT;
    private int BUFFER_SIZE = AudioRecord.getMinBufferSize(SAMPLE_RATE, CHANNEL_CONFIG, AUDIO_FORMAT);

    // Runnable management
    private volatile boolean record = false;
    private volatile boolean isRecording = false;

    private final Runnable runnable = new Runnable() {
        @Override
        public void run() {
            isRecording = true;

            // Set up Google cloud streaming

            // Repeatedly push audio samples to stream
            while (record) {
                // Read audio data into new byte array
                short[] audio = new short[BUFFER_SIZE];
                recorder.read(audio, 0, BUFFER_SIZE);

                String transcript = "Transcript!\t";

                // push data into stream
                try {
                    eventSink.success(transcript);
                } catch (IllegalArgumentException e) {
                    eventSink.error("-12", "InvalidData", e);
                } catch (NullPointerException e) {
                    eventSink.error("-11", "NullPointer", e);
                }
            }

            isRecording = false;
        }
    };

    @Override
    public void onListen(Object args, EventChannel.EventSink eventSink) {
        if (isRecording) return;

        ArrayList<String> config = (ArrayList<String>) args;
        switch (config.size()) {
            case 0:
                eventSink.error("-21", "InvalidArguments", null);
            case 1:
                api_key = config.get(0);
                break;
            default:
                eventSink.error("-22", "TooManyArguments", null);
        }

        this.eventSink = eventSink;

        // Try to initialize and start the recorder
        recorder = new AudioRecord(AUDIO_SOURCE, SAMPLE_RATE, CHANNEL_CONFIG, AUDIO_FORMAT, BUFFER_SIZE);
        if (recorder.getState() != AudioRecord.STATE_INITIALIZED) eventSink.error("-1", "PlatformError", null);
        recorder.startRecording();

        // Start runnable
        record = true;
        new Thread(runnable).start();
    }

    @Override
    public void onCancel(Object args) {
        // Stop runnable
        record = false;

        // Stop and reset audio recorder
        if (recorder != null) {
            recorder.stop();
            recorder.release();
            recorder = null;
        }
    }
}
